@@ -2,8 +2,6 @@
 use std::pin::Pin;
 use std::task::{Context, Poll};
 
-use bytes::Bytes;
-use futures_core::Stream;
 use h2::RecvStream;
 
 mod dispatcher;
@@ -11,7 +9,7 @@ mod service;
 
 pub use self::dispatcher::Dispatcher;
 pub use self::service::H2Service;
-use crate::http::error::PayloadError;
+use crate::{http::error::PayloadError, util::Bytes, Stream};
 
 /// H2 receive stream
 #[derive(Debug)]
@@ -40,7 +38,7 @@ impl Stream for Payload {
                 if let Err(err) = this.pl.flow_control().release_capacity(len) {
                     Poll::Ready(Some(Err(err.into())))
                 } else {
-                    Poll::Ready(Some(Ok(chunk)))
+                    Poll::Ready(Some(Ok(Bytes::copy_from_slice(&chunk[..]))))
                 }
             }
             Poll::Ready(Some(Err(err))) => Poll::Ready(Some(Err(err.into()))),
