@@ -5,8 +5,8 @@ use std::{
 use crate::http::Response;
 use crate::router::{IntoPattern, ResourceDef};
 use crate::service::boxed::{self, BoxService, BoxServiceFactory};
-use crate::service::{apply, apply_fn_factory, pipeline_factory};
-use crate::service::{IntoServiceFactory, Service, ServiceFactory, Transform};
+use crate::service::{pipeline_factory, PipelineFactory};
+use crate::service::{Identity, IntoServiceFactory, Service, ServiceFactory, Transform};
 use crate::util::{Either, Extensions, Ready};
 
 use super::dev::{insert_slesh, WebServiceConfig, WebServiceFactory};
@@ -18,7 +18,7 @@ use super::request::WebRequest;
 use super::responder::Responder;
 use super::response::WebResponse;
 use super::route::{IntoRoutes, Route, RouteService};
-use super::types::Data;
+use super::{app::Filter, app::Stack, types::Data};
 
 type HttpService<Err: ErrorRenderer> =
     BoxService<WebRequest<Err>, WebResponse, Err::Container>;
@@ -47,27 +47,25 @@ type HttpNewService<Err: ErrorRenderer> =
 ///
 /// If no matching route could be found, *405* response code get returned.
 /// Default behavior could be overriden with `default_resource()` method.
-pub struct Resource<Err: ErrorRenderer, T = ResourceEndpoint<Err>> {
-    endpoint: T,
+pub struct Resource<Err: ErrorRenderer, M = Identity, T = Filter<Err>> {
+    middleware: M,
+    filter: PipelineFactory<T>,
     rdef: Vec<String>,
     name: Option<String>,
     routes: Vec<Route<Err>>,
     data: Option<Extensions>,
     guards: Vec<Box<dyn Guard>>,
     default: Rc<RefCell<Option<Rc<HttpNewService<Err>>>>>,
-    factory_ref: Rc<RefCell<Option<ResourceFactory<Err>>>>,
 }
 
 impl<Err: ErrorRenderer> Resource<Err> {
     pub fn new<T: IntoPattern>(path: T) -> Resource<Err> {
-        let fref = Rc::new(RefCell::new(None));
-
         Resource {
             routes: Vec::new(),
             rdef: path.patterns(),
             name: None,
-            endpoint: ResourceEndpoint::new(fref.clone()),
-            factory_ref: fref,
+            middleware: Identity,
+            filter: pipeline_factory(Filter::new()),
             guards: Vec::new(),
             data: None,
             default: Rc::new(RefCell::new(None)),
@@ -75,12 +73,12 @@ impl<Err: ErrorRenderer> Resource<Err> {
     }
 }
 
-impl<Err, T> Resource<Err, T>
+impl<Err, M, T> Resource<Err, M, T>
 where
     T: ServiceFactory<
         Config = (),
         Request = WebRequest<Err>,
-        Response = WebResponse,
+        Response = WebRequest<Err>,
         Error = Err::Container,
         InitError = (),
     >,
@@ -163,9 +161,9 @@ where
     /// # async fn post_handler() -> web::HttpResponseBuilder { web::HttpResponse::Ok() }
     /// # async fn delete_handler() -> web::HttpResponseBuilder { web::HttpResponse::Ok() }
     /// ```
-    pub fn route<U>(mut self, route: U) -> Self
+    pub fn route<R>(mut self, route: R) -> Self
     where
-        U: IntoRoutes<Err>,
+        R: IntoRoutes<Err>,
     {
         for route in route.routes() {
             self.routes.push(route);
@@ -198,14 +196,14 @@ where
     ///           ));
     /// }
     /// ```
-    pub fn data<U: 'static>(self, data: U) -> Self {
+    pub fn data<D: 'static>(self, data: D) -> Self {
         self.app_data(Data::new(data))
     }
 
     /// Set or override application data.
     ///
     /// This method overrides data stored with [`App::app_data()`](#method.app_data)
-    pub fn app_data<U: 'static>(mut self, data: U) -> Self {
+    pub fn app_data<D: 'static>(mut self, data: D) -> Self {
         if self.data.is_none() {
             self.data = Some(Extensions::new());
         }
@@ -246,46 +244,39 @@ where
     /// Register request filter.
     ///
     /// This is similar to `App's` filters, but filter get invoked on resource level.
-    pub fn filter<F>(
+    pub fn filter<U, F>(
         self,
         filter: F,
     ) -> Resource<
         Err,
+        M,
         impl ServiceFactory<
             Config = (),
             Request = WebRequest<Err>,
-            Response = WebResponse,
+            Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
         >,
     >
     where
-        F: ServiceFactory<
+        U: ServiceFactory<
             Config = (),
             Request = WebRequest<Err>,
-            Response = Either<WebRequest<Err>, WebResponse>,
+            Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
         >,
+        F: IntoServiceFactory<U>,
     {
-        let ep = self.endpoint;
-        let endpoint =
-            pipeline_factory(filter).and_then_apply_fn(ep, move |result, srv| {
-                match result {
-                    Either::Left(req) => Either::Left(srv.call(req)),
-                    Either::Right(res) => Either::Right(Ready::Ok(res)),
-                }
-            });
-
         Resource {
-            endpoint,
+            filter: self.filter.and_then(filter.into_factory()),
+            middleware: self.middleware,
             rdef: self.rdef,
             name: self.name,
             guards: self.guards,
             routes: self.routes,
             default: self.default,
             data: self.data,
-            factory_ref: self.factory_ref,
         }
     }
 
@@ -296,116 +287,32 @@ where
     /// type (i.e modify response's body).
     ///
     /// **Note**: middlewares get called in opposite order of middlewares registration.
-    pub fn wrap<M>(
-        self,
-        mw: M,
-    ) -> Resource<
-        Err,
-        impl ServiceFactory<
-            Config = (),
-            Request = WebRequest<Err>,
-            Response = WebResponse,
-            Error = Err::Container,
-            InitError = (),
-        >,
-    >
-    where
-        M: Transform<
-            T::Service,
-            Request = WebRequest<Err>,
-            Response = WebResponse,
-            Error = Err::Container,
-            InitError = (),
-        >,
-    {
+    pub fn wrap<U>(self, mw: U) -> Resource<Err, Stack<M, U>, T> {
         Resource {
-            endpoint: apply(mw, self.endpoint),
+            middleware: Stack::new(self.middleware, mw),
+            filter: self.filter,
             rdef: self.rdef,
             name: self.name,
             guards: self.guards,
             routes: self.routes,
             default: self.default,
             data: self.data,
-            factory_ref: self.factory_ref,
-        }
-    }
-
-    /// Register a resource middleware function.
-    ///
-    /// This function accepts instance of `WebRequest` type and
-    /// mutable reference to the next middleware in chain.
-    ///
-    /// This is similar to `App's` middlewares, but middleware get invoked on resource level.
-    /// Resource level middlewares are not allowed to change response
-    /// type (i.e modify response's body).
-    ///
-    /// ```rust
-    /// use ntex::service::Service;
-    /// use ntex::web::{self, App};
-    /// use ntex::http::header::{CONTENT_TYPE, HeaderValue};
-    ///
-    /// async fn index() -> &'static str {
-    ///     "Welcome!"
-    /// }
-    ///
-    /// fn main() {
-    ///     let app = App::new().service(
-    ///         web::resource("/index.html")
-    ///             .wrap_fn(|req, srv| {
-    ///                 let fut = srv.call(req);
-    ///                 async {
-    ///                     let mut res = fut.await?;
-    ///                     res.headers_mut().insert(
-    ///                        CONTENT_TYPE, HeaderValue::from_static("text/plain"),
-    ///                     );
-    ///                     Ok(res)
-    ///                 }
-    ///             })
-    ///             .route(web::get().to(index)));
-    /// }
-    /// ```
-    pub fn wrap_fn<F, R>(
-        self,
-        mw: F,
-    ) -> Resource<
-        Err,
-        impl ServiceFactory<
-            Config = (),
-            Request = WebRequest<Err>,
-            Response = WebResponse,
-            Error = Err::Container,
-            InitError = (),
-        >,
-    >
-    where
-        F: Fn(WebRequest<Err>, &T::Service) -> R + Clone,
-        R: Future<Output = Result<WebResponse, Err::Container>>,
-    {
-        Resource {
-            endpoint: apply_fn_factory(self.endpoint, mw),
-            rdef: self.rdef,
-            name: self.name,
-            guards: self.guards,
-            routes: self.routes,
-            default: self.default,
-            data: self.data,
-            factory_ref: self.factory_ref,
         }
     }
 
     /// Default service to be used if no matching route could be found.
     /// By default *405* response get returned. Resource does not use
     /// default handler from `App` or `Scope`.
-    pub fn default_service<F, U>(mut self, f: F) -> Self
+    pub fn default_service<F, S>(mut self, f: F) -> Self
     where
-        F: IntoServiceFactory<U>,
-        U: ServiceFactory<
+        F: IntoServiceFactory<S>,
+        S: ServiceFactory<
                 Config = (),
                 Request = WebRequest<Err>,
                 Response = WebResponse,
                 Error = Err::Container,
             > + 'static,
-        U::InitError: fmt::Debug,
+        S::InitError: fmt::Debug,
     {
         // create and configure default resource
         self.default = Rc::new(RefCell::new(Some(Rc::new(boxed::factory(
@@ -418,15 +325,21 @@ where
     }
 }
 
-impl<Err, T> WebServiceFactory<Err> for Resource<Err, T>
+impl<Err, M, T> WebServiceFactory<Err> for Resource<Err, M, T>
 where
     T: ServiceFactory<
             Config = (),
             Request = WebRequest<Err>,
-            Response = WebResponse,
+            Response = WebRequest<Err>,
             Error = Err::Container,
             InitError = (),
         > + 'static,
+    M: Transform<ResourceService<T::Service, Err>> + 'static,
+    M::Service: Service<
+        Request = WebRequest<Err>,
+        Response = WebResponse,
+        Error = Err::Container,
+    >,
     Err: ErrorRenderer,
 {
     fn register(mut self, config: &mut WebServiceConfig<Err>) {
@@ -447,45 +360,193 @@ where
         if let Some(ref mut ext) = self.data {
             config.set_service_data(ext);
         }
-        config.register_service(rdef, guards, self, None)
-    }
-}
 
-impl<Err, T> IntoServiceFactory<T> for Resource<Err, T>
-where
-    T: ServiceFactory<
-        Config = (),
-        Request = WebRequest<Err>,
-        Response = WebResponse,
-        Error = Err::Container,
-        InitError = (),
-    >,
-    Err: ErrorRenderer,
-{
-    fn into_factory(self) -> T {
-        *self.factory_ref.borrow_mut() = Some(ResourceFactory {
+        let router_factory = ResourceRouterFactory {
             routes: self.routes,
             data: self.data.map(Rc::new),
             default: self.default,
-        });
+        };
 
-        self.endpoint
+        config.register_service(
+            rdef,
+            guards,
+            ResourceServiceFactory {
+                middleware: Rc::new(self.middleware),
+                filter: self.filter,
+                routing: router_factory,
+            },
+            None,
+        )
     }
 }
 
-struct ResourceFactory<Err: ErrorRenderer> {
+impl<Err, M, T> IntoServiceFactory<ResourceServiceFactory<Err, M, PipelineFactory<T>>>
+    for Resource<Err, M, T>
+where
+    T: ServiceFactory<
+            Config = (),
+            Request = WebRequest<Err>,
+            Response = WebRequest<Err>,
+            Error = Err::Container,
+            InitError = (),
+        > + 'static,
+    M: Transform<ResourceService<T::Service, Err>> + 'static,
+    M::Service: Service<
+        Request = WebRequest<Err>,
+        Response = WebResponse,
+        Error = Err::Container,
+    >,
+    Err: ErrorRenderer,
+{
+    fn into_factory(self) -> ResourceServiceFactory<Err, M, PipelineFactory<T>> {
+        let router_factory = ResourceRouterFactory {
+            routes: self.routes,
+            data: self.data.map(Rc::new),
+            default: self.default,
+        };
+
+        ResourceServiceFactory {
+            middleware: Rc::new(self.middleware),
+            filter: self.filter,
+            routing: router_factory,
+        }
+    }
+}
+
+/// Resource service
+pub struct ResourceServiceFactory<Err: ErrorRenderer, M, T> {
+    middleware: Rc<M>,
+    filter: T,
+    routing: ResourceRouterFactory<Err>,
+}
+
+impl<Err, M, F> ServiceFactory for ResourceServiceFactory<Err, M, F>
+where
+    M: Transform<ResourceService<F::Service, Err>> + 'static,
+    M::Service: Service<
+        Request = WebRequest<Err>,
+        Response = WebResponse,
+        Error = Err::Container,
+    >,
+    F: ServiceFactory<
+            Config = (),
+            Request = WebRequest<Err>,
+            Response = WebRequest<Err>,
+            Error = Err::Container,
+            InitError = (),
+        > + 'static,
+    Err: ErrorRenderer,
+{
+    type Config = ();
+    type Request = WebRequest<Err>;
+    type Response = WebResponse;
+    type Error = Err::Container;
+    type Service = M::Service;
+    type InitError = ();
+    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+
+    fn new_service(&self, _: ()) -> Self::Future {
+        let filter_fut = self.filter.new_service(());
+        let routing_fut = self.routing.new_service(());
+        let middleware = self.middleware.clone();
+        Box::pin(async move {
+            Ok(middleware.new_transform(ResourceService {
+                filter: filter_fut.await?,
+                routing: Rc::new(routing_fut.await?),
+            }))
+        })
+    }
+}
+
+pub struct ResourceService<F, Err: ErrorRenderer> {
+    filter: F,
+    routing: Rc<ResourceRouter<Err>>,
+}
+
+impl<F, Err> Service for ResourceService<F, Err>
+where
+    F: Service<
+        Request = WebRequest<Err>,
+        Response = WebRequest<Err>,
+        Error = Err::Container,
+    >,
+    Err: ErrorRenderer,
+{
+    type Request = WebRequest<Err>;
+    type Response = WebResponse;
+    type Error = Err::Container;
+    type Future = ResourceServiceResponse<F, Err>;
+
+    #[inline]
+    fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
+        let ready1 = self.filter.poll_ready(cx)?.is_ready();
+        let ready2 = self.routing.poll_ready(cx)?.is_ready();
+        if ready1 && ready2 {
+            Poll::Ready(Ok(()))
+        } else {
+            Poll::Pending
+        }
+    }
+
+    fn call(&self, req: WebRequest<Err>) -> Self::Future {
+        ResourceServiceResponse {
+            filter: self.filter.call(req),
+            routing: self.routing.clone(),
+            endpoint: None,
+        }
+    }
+}
+
+pin_project_lite::pin_project! {
+    pub struct ResourceServiceResponse<F: Service, Err: ErrorRenderer> {
+        #[pin]
+        filter: F::Future,
+        routing: Rc<ResourceRouter<Err>>,
+        endpoint: Option<<ResourceRouter<Err> as Service>::Future>,
+    }
+}
+
+impl<F, Err> Future for ResourceServiceResponse<F, Err>
+where
+    F: Service<
+        Request = WebRequest<Err>,
+        Response = WebRequest<Err>,
+        Error = Err::Container,
+    >,
+    Err: ErrorRenderer,
+{
+    type Output = Result<WebResponse, Err::Container>;
+
+    fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
+        let this = self.as_mut().project();
+
+        if let Some(fut) = this.endpoint.as_mut() {
+            Pin::new(fut).poll(cx)
+        } else {
+            let res = if let Poll::Ready(res) = this.filter.poll(cx) {
+                res?
+            } else {
+                return Poll::Pending;
+            };
+            *this.endpoint = Some(this.routing.call(res));
+            self.poll(cx)
+        }
+    }
+}
+
+struct ResourceRouterFactory<Err: ErrorRenderer> {
     routes: Vec<Route<Err>>,
     data: Option<Rc<Extensions>>,
     default: Rc<RefCell<Option<Rc<HttpNewService<Err>>>>>,
 }
 
-impl<Err: ErrorRenderer> ServiceFactory for ResourceFactory<Err> {
+impl<Err: ErrorRenderer> ServiceFactory for ResourceRouterFactory<Err> {
     type Config = ();
     type Request = WebRequest<Err>;
     type Response = WebResponse;
     type Error = Err::Container;
     type InitError = ();
-    type Service = ResourceService<Err>;
+    type Service = ResourceRouter<Err>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
@@ -500,7 +561,7 @@ impl<Err: ErrorRenderer> ServiceFactory for ResourceFactory<Err> {
                 None
             };
 
-            Ok(ResourceService {
+            Ok(ResourceRouter {
                 routes,
                 data,
                 default,
@@ -509,13 +570,13 @@ impl<Err: ErrorRenderer> ServiceFactory for ResourceFactory<Err> {
     }
 }
 
-pub struct ResourceService<Err: ErrorRenderer> {
+struct ResourceRouter<Err: ErrorRenderer> {
     routes: Vec<RouteService<Err>>,
     data: Option<Rc<Extensions>>,
     default: Option<HttpService<Err>>,
 }
 
-impl<Err: ErrorRenderer> Service for ResourceService<Err> {
+impl<Err: ErrorRenderer> Service for ResourceRouter<Err> {
     type Request = WebRequest<Err>;
     type Response = WebResponse;
     type Error = Err::Container;
@@ -549,51 +610,28 @@ impl<Err: ErrorRenderer> Service for ResourceService<Err> {
     }
 }
 
-#[doc(hidden)]
-pub struct ResourceEndpoint<Err: ErrorRenderer> {
-    factory: Rc<RefCell<Option<ResourceFactory<Err>>>>,
-}
-
-impl<Err: ErrorRenderer> ResourceEndpoint<Err> {
-    fn new(factory: Rc<RefCell<Option<ResourceFactory<Err>>>>) -> Self {
-        ResourceEndpoint { factory }
-    }
-}
-
-impl<Err: ErrorRenderer> ServiceFactory for ResourceEndpoint<Err> {
-    type Config = ();
-    type Request = WebRequest<Err>;
-    type Response = WebResponse;
-    type Error = Err::Container;
-    type InitError = ();
-    type Service = ResourceService<Err>;
-    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
-
-    fn new_service(&self, _: ()) -> Self::Future {
-        self.factory.borrow_mut().as_mut().unwrap().new_service(())
-    }
-}
-
 #[cfg(test)]
 mod tests {
-    use std::time::Duration;
-
     use crate::http::header::{self, HeaderValue};
     use crate::http::{Method, StatusCode};
-    use crate::rt::time::sleep;
+    use crate::time::{sleep, Millis};
     use crate::web::middleware::DefaultHeaders;
-    use crate::web::request::WebRequest;
     use crate::web::test::{call_service, init_service, TestRequest};
-    use crate::web::{self, guard, App, DefaultError, HttpResponse};
-    use crate::{fn_service, util::Either, Service};
+    use crate::web::{
+        self, guard, request::WebRequest, App, DefaultError, HttpResponse,
+    };
+    use crate::{service::fn_service, util::Ready};
 
     #[crate::rt_test]
     async fn test_filter() {
+        let filter = std::rc::Rc::new(std::cell::Cell::new(false));
+        let filter2 = filter.clone();
         let srv = init_service(
             App::new().service(
                 web::resource("/test")
-                    .filter(fn_service(|req: WebRequest<_>| async move {
-                        Ok(Either::Right(req.into_response(HttpResponse::NotFound())))
+                    .filter(fn_service(move |req: WebRequest<_>| {
+                        filter2.set(true);
+                        Ready::Ok(req)
                     }))
                     .route(web::get().to(|| async { HttpResponse::Ok() })),
             ),
@@ -601,7 +639,8 @@ mod tests {
         .await;
         let req = TestRequest::with_uri("/test").to_request();
         let resp = call_service(&srv, req).await;
-        assert_eq!(resp.status(), StatusCode::NOT_FOUND);
+        assert_eq!(resp.status(), StatusCode::OK);
+        assert!(filter.get());
     }
 
     #[crate::rt_test]
@@ -628,41 +667,11 @@ mod tests {
         );
     }
 
-    #[crate::rt_test]
-    async fn test_middleware_fn() {
-        let srv = init_service(
-            App::new().service(
-                web::resource("/test")
-                    .wrap_fn(|req, srv| {
-                        let fut = srv.call(req);
-                        async {
-                            fut.await.map(|mut res| {
-                                res.headers_mut().insert(
-                                    header::CONTENT_TYPE,
-                                    HeaderValue::from_static("0001"),
-                                );
-                                res
-                            })
-                        }
-                    })
-                    .route(web::get().to(|| async { HttpResponse::Ok() })),
-            ),
-        )
-        .await;
-        let req = TestRequest::with_uri("/test").to_request();
-        let resp = call_service(&srv, req).await;
-        assert_eq!(resp.status(), StatusCode::OK);
-        assert_eq!(
-            resp.headers().get(header::CONTENT_TYPE).unwrap(),
-            HeaderValue::from_static("0001")
-        );
-    }
-
     #[crate::rt_test]
     async fn test_to() {
         let srv =
             init_service(App::new().service(web::resource("/test").to(|| async {
-                sleep(Duration::from_millis(100)).await;
+                sleep(Millis(100)).await;
                 HttpResponse::Ok()
             })))
             .await;
